/*
 * Clone all Blocks from an input document.
 * 
 * Required software: PPS 9
 * Required data: PDF document with Blocks
 */

package com.pdflib.cookbook.pdflib.blocks;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class clone_blocks {
    public static void main(String argv[]) {
        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";

        /* By default annotations are also imported. In some cases this
         * requires the Noto fonts for creating annotation appearance streams.
         * We therefore set the searchpath to also point to the font directory.
         */
        String fontpath = "../resource/font";

        String outfile = "clone_blocks.pdf";
        String infile = "businesscard_blocks.pdf";
        String title = "Clone PDFlib Blocks";
        int exitcode = 0;

        pdflib p = null;

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            p.set_option("searchpath={" + fontpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            /* Open a PDF containing Blocks */
            int indoc = p.open_pdi_document(infile, "");
            if (indoc == -1)
                throw new Exception("Error: " + p.get_errmsg());

            /* Loop over all pages of the input document */
            int pagecount = (int) p.pcos_get_number(indoc, "length:pages");
            for (int pageno = 1; pageno <= pagecount; pageno++)
            {
                
                int page = p.open_pdi_page(indoc, pageno, "");
                if (page == -1)
                    throw new Exception("Error: " + p.get_errmsg());
    
                /* Page size may be adjusted by fit_pdi_page() */
                p.begin_page_ext(0, 0, "width=a4.width height=a4.height");
    
                /* Copy all Blocks from the page */
                if (p.process_pdi(indoc, 0,
                    "action=copyallblocks block={pagenumber=" + pageno + "}") == -1)
                    throw new Exception("Error: " + p.get_errmsg());
    
                /* Import the page content itself */
                p.fit_pdi_page(page, 0, 0, "adjustpage");
    
                p.end_page_ext("");
            }

            p.end_document("");
        }
        catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        }
        catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        }
        finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
