/*
 * Form text field input format:
 * Format the data entered in a form field of type "textfield" according to
 * the rules defined.
 * 
 * Create a text field for displaying a price and format the input as
 * "x,xxx.xx" with the text " Euro" appended. 
 * 
 * Required software: PDFlib/PDFlib+PDI/PPS 10
 * Required data: none
 */
package com.pdflib.cookbook.pdflib.form_fields;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class form_textfield_input_format
{
    public static void main (String argv[])
    {
        /* This is where the data files are. Adjust as necessary */
        String searchpath = "../input";
        String outfile = "form_textfield_input_format.pdf";
        String title = "Form Text Field Input Format";
        
        pdflib p = null;
        int exitcode = 0;

        String optlist;
        int font, monospaced_font, format_action;
        double fontsize, ascender, descender;
        double width=160, height=30, llx = 10, lly = 700;

        try {
            p = new pdflib();

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            p.set_option("searchpath={" + searchpath + "}");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            font = p.load_font("NotoSerif-Regular", "winansi", 
                                          "simplefont nosubsetting");
            if (font == -1)
                throw new Exception("Error: " + p.get_errmsg());
            
            /* Start page */
            p.begin_page_ext(0, 0, " width=a4.width height=a4.height");
            
            /* -----------------------------------------------
             * Format a form text field for displaying a price
             * -----------------------------------------------
             *
             * For a correct formatting of the number, define an action which
             * formats the number as "x,xxx.xx", e.g. "1,200.50", and appends the
             * text " Euro". 
             */
            format_action = p.create_action("JavaScript",
                "script={AFNumber_Format(2, 0, 0, 0, \" Euro\", false); }");
                   
            /* Create a form field of type "textfield" called "price" with the
             * JavaScript action supplied in the "action" option 
             */
            optlist = "backgroundcolor={lightblue} bordercolor={gray 0} " +
                "maxchar=10 scrollable=false tooltip={Enter the price} " +
                "font=" + font + " fontsize=24" + 
                " action={format=" + format_action + "}";
            
            width=250;
            p.create_field(llx, lly, llx + width, lly + height, "price",
                "textfield", optlist);
            lly-=40;
            
            p.fit_textline("Enter a price, e.g. 150.95. A maximum of 10 digits " +
                "is allowed to be entered.", llx, lly, 
                "fontname=NotoSerif-Regular fontsize=12");
            p.fit_textline("After pressing \"Tab\" or \"Enter\", the price is " +
                "formatted as \"x,xxx.xx\" with the text \" Euro\" appended.", 
                llx, lly-=20,
                "fontname=NotoSerif-Regular fontsize=12");
            
            p.end_page_ext("");
            
            p.end_document("");

        } catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname()
                + ": " + e.get_errmsg());
            exitcode = 1;
        } catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        } finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
