/*
 * Overprinting text:
 * Create text which will overprint other page contents instead of knocking
 * it out
 *
 * Create an extended graphics state with the options
 * "overprintfill=true overprintmode=1".
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 10
 * Required data: none
 */
package com.pdflib.cookbook.pdflib.graphics;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class overprinting_text {
    public static void main(String argv[]) {
        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";
        String outfile = "overprinting_text.pdf";
        String title = "Overprinting Text";
        int exitcode = 0;

        pdflib p = null;

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            /* Start page */
            p.begin_page_ext(400, 300, "");

            /* Draw a red background rectangle */
            p.setcolor("fill", "cmyk", 0, 1, 1, 0);
            p.rect(0, 0, 400, 300);
            p.fill();

            /* Save the current graphics state */
            p.save();

            /* Create an extended graphics state */
            int gstate = p.create_gstate("overprintfill overprintmode=1");

            /* Apply the extended graphics state */
            p.set_gstate(gstate);

            /*
             * Show some text which will overprint other page contents according
             * to the graphics state set
             */
            String fontopts = "fontname=NotoSerif-Regular fontsize=36";

            p.setcolor("fill", "cmyk", 0, 0, 0, 1);
            p.fit_textline("overprinting text", 20, 200, fontopts);

            /* Restore the current graphics state */
            p.restore();

            /*
             * Show some text. It will not be overprinting since the old
             * graphics state has been restored with no overprinting set.
             */
            p.setcolor("fill", "cmyk", 0, 0, 0, 1);
            p.fit_textline("text not overprinting", 20, 100, fontopts);

            p.end_page_ext("");

            p.end_document("");
        }
        catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        }
        catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        }
        finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
