/*
 * PDF merge starter:
 * Merge pages from multiple PDF documents (including annotations)
 *
 * Required software: PDFlib+PDI/PPS 9
 * Required data: PDF documents
 */
package com.pdflib.cookbook.pdflib.pdf_import;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class starter_pdfmerge
{
    public static void main (String argv[])
    {
        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";

        /* By default annotations are also imported. In some cases this
         * requires the Noto fonts for creating annotation appearance streams.
         * We therefore set the searchpath to also point to the font directory.
         */
        String fontpath = "../resource/font";
        String outfile = "starter_pdfmerge.pdf";
        String title = "Starter PDF Merge";

        pdflib p = null;
        int exitcode = 0;
        String pdffiles[] =
        {
            "markup_annotations_input.pdf",
            "PLOP-datasheet.pdf",
            "pCOS-datasheet.pdf"
        };
        int i;

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            p.set_option("searchpath={" + fontpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            for (i=0; i < pdffiles.length; i++)
            {
            int indoc, endpage, pageno, page;

            /* Open the input PDF */
            indoc = p.open_pdi_document(pdffiles[i], "");
            if (indoc == -1)
            {
                System.err.println("Error: " + p.get_errmsg());
                continue;
            }

            endpage = (int) p.pcos_get_number(indoc, "length:pages");

            /* Loop over all pages of the input document */
            for (pageno = 1; pageno <= endpage; pageno++)
            {
                page = p.open_pdi_page(indoc, pageno, "");

                if (page == -1)
                {
                    System.err.println("Error: " + p.get_errmsg());
                    continue;
                }
                /* Page size may be adjusted by fit_pdi_page(). */
                p.begin_page_ext(0, 0, "width=a4.width height=a4.height");

                /* Create a bookmark with the file name */
                if (pageno == 1)
                    p.create_bookmark(pdffiles[i], "");

                /* Place the imported page on the output page, and
                 * adjust the page size. If the page contains annotations
                 * these are also imported.
                 */
                p.fit_pdi_page(page, 0, 0, "adjustpage");
                p.close_pdi_page(page);

                p.end_page_ext("");
            }
            p.close_pdi_document(indoc);
            }

            p.end_document("");

        } catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        } catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        } finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
