/*
 * PDF/A extension schema with type:
 * Demonstrate the use of a complex XMP extension schema as defined in PDF/A-1
 * 
 * Create a PDF/A document and use the "metadata" option of begin_document() to
 * supply an XMP file containing a complex XMP extension schema which requires a
 * custom type definition.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 10
 * Required data: XMP file
 */
package com.pdflib.cookbook.pdflib.pdfa;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class xmp_extension_schema_with_type_pdfa2b {
    public static void main(String argv[]) {
        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";
        String outfile = "xmp_extension_schema_with_type_pdfa2b.pdf";
        String title = "PDF/A Extension Schema with Type";

        pdflib p = null;

        String xmpfile = "machine_extension_schema_2.xmp";
        int font;
        int exitcode = 0;

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            /*
             * Start a PDF/A document with an XMP file supplied, containing an
             * XMP extension schema. The XMP metadata will be read from the file
             * and embedded in the document. PDFlib will merge several
             * internally generated entries into the user-supplied XMP, e.g.
             * xmp:CreateDate.
             */
            if (p.begin_document(outfile,
                    "pdfa=PDF/A-2b metadata={filename={" + xmpfile + "}}")
                                                                        == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            p.begin_page_ext(0, 0, "width=a4.height height=a4.width");

            font = p.load_font("NotoSerif-Regular", "unicode", "");

            if (font == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.setfont(font, 12);

            p.fit_textline(
                    "An XMP extension schema is read from an XMP file and "
                            + "the XMP metadata is embedded in the document.",
                    50, 400, "");

            p.end_page_ext("");

            p.end_document("");

        }
        catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        }
        catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        }
        finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
